----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:27:42 06/14/2011 
-- Design Name: 
-- Module Name:    SAR C2C 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_arith.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity SAR_C2C_NEXYS3 is
	port (
		CLK1: IN STD_LOGIC;				            -- clock (naturally)
   	CLK2: IN STD_LOGIC;				            -- clock (naturally) not used
		btn: IN STD_LOGIC_VECTOR (3 DOWNTO 0);	   -- Buttons on Basys
		seg: OUT STD_LOGIC_VECTOR (6 DOWNTO 0);	-- Segments
		an: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);	   -- Digit
		dp: OUT STD_LOGIC;				               -- dot in Segment
		led: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);	-- Segments
      sw: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
      JA: OUT STD_LOGIC_VECTOR (7 downto 0);      -- Output stream
		JB1: OUT STD_LOGIC_VECTOR (4 downto 0);        --    NEXYS3 
		JB2: In STD_LOGIC_VECTOR (1 downto 0);        --    NEXYS3 
      JC: OUT STD_LOGIC_VECTOR (7 downto 0);      -- Output stream
		TxD: OUT STD_LOGIC;        --    NEXYS3 
		RxD: IN STD_LOGIC          --    NEXYS3 
	);
end SAR_C2C_NEXYS3;

-- JB2(0) Analog input
-- JB2(1) DAC output
-- JB1(3) Compare out

-- switch (2 downto 0) control pulsewidth and sample rate
-- count8 per cycle
-- 000 100us pulse 10  kHz -> 80kHz count8
-- 001  50us pulse 20  kHz
-- 010  20us pulse 50  kHz
-- 011  10us pulse 100 kHz
-- 100   5us pulse 200 kHz
-- 101   2us pulse 500 kHz
-- 110   1us pulse 1   MHz
-- 111 100ns pulse 10  MHz

-- switch (4 downto 3) operation
-- 00  SAR operation
-- 01  sawtooth down DAC
-- 10  sawtooth up DAC
-- 11  sine signal DAC

-- switch (5 downto 6) control number of bits
-- 00  (32)  16
-- 01  (16)  12 
-- 10   (8)  10
-- 11   (2)  8

-- switch (sw7) = 1 controls enabling inverted data for capacitive C2C

architecture Behavioral of SAR_C2C_NEXYS3 is
 
 -- Component Declaration
    COMPONENT SegCntrl
      PORT(
		            CLK1: IN STD_LOGIC;				            -- clock (naturally)
                Digit0: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		          Digit1: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		          Digit2: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		          Digit3: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		           reset: IN STD_LOGIC;
		             seg: OUT STD_LOGIC_VECTOR (6 DOWNTO 0);	-- Segments
		              an: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);	   -- Digit
		              dp: OUT STD_LOGIC				               -- dot in Segment
                  );
    END COMPONENT;

    COMPONENT CLK_DIV 
    Port ( CLK : in  STD_LOGIC;
	        reset : in STD_LOGIC;
           EN_4 : out  STD_LOGIC;
           EN_16 : out  STD_LOGIC;
           EN_32 : out  STD_LOGIC;
           EN_64 : out  STD_LOGIC;
           EN_128 : out  STD_LOGIC;
           EN_256 : out  STD_LOGIC;
           EN_512 : out  STD_LOGIC;
           EN_1024 : out  STD_LOGIC;
           EN_2048 : out  STD_LOGIC;
           EN_4096 : out  STD_LOGIC;
           EN_16K : out  STD_LOGIC;
           EN_65K : out  STD_LOGIC;
			  EN_1M : out  STD_LOGIC);
    end COMPONENT;

  COMPONENT uart
   port(
      clk, reset: in std_logic;
      rd_uart, wr_uart: in std_logic;
      rx: in std_logic;
      w_data: in std_logic_vector(7 downto 0);
      tx_full, rx_empty: out std_logic;
      r_data: out std_logic_vector(7 downto 0);
      tx: out std_logic
   );
   end COMPONENT;


SIGNAL Code, BCode, CCode, C1Code, HCode, TCode, STCode: STD_LOGIC_VECTOR (15 downto 0);
SIGNAL SCode, MZero, MOne: STD_LOGIC_VECTOR (17 downto 0);

SIGNAL D: STD_LOGIC;
SIGNAL SH: STD_LOGIC;
SIGNAL SSAR: STD_LOGIC;
SIGNAL Count8: STD_LOGIC_VECTOR (3 downto 0);
SIGNAL SigS: STD_LOGIC_VECTOR (1 downto 0);
SIGNAL muxs: STD_LOGIC_VECTOR (2 downto 0);
SIGNAL JAX: STD_LOGIC_VECTOR (5 downto 0);
SIGNAL reset: STD_LOGIC;
SIGNAL ENX, EN_4, EN_16, EN_32, EN_64, EN_128, EN_256, EN_4096, EN_16K, EN_65K, EN_1M: STD_LOGIC;
SIGNAL EN_512, EN_1024, EN_2048, EN_T, EN_X: STD_LOGIC;
SIGNAL Sel: STD_LOGIC_VECTOR (2 downto 0);

SIGNAL wr_uart,tx_full, rx_empty: STD_LOGIC;
SIGNAL w_data,RD_data: STD_LOGIC_VECTOR(7 downto 0);
SIGNAL i_data: STD_LOGIC_VECTOR(4 downto 0);
SIGNAL ST: STD_LOGIC_VECTOR(4 downto 0);
SIGNAL decode: STD_LOGIC_VECTOR(2 downto 0);
SIGNAL trigger: STD_LOGIC_VECTOR(2 downto 0);
SIGNAL Valid: STD_LOGIC;
SIGNAL TXI: STD_LOGIC;
SIGNAL MaxCount: STD_LOGIC_VECTOR(8 downto 0);
SIGNAL douta: STD_LOGIC_VECTOR(11 downto 0);
SIGNAL addra: STD_LOGIC_VECTOR(13 downto 0);
SIGNAL wea: STD_LOGIC_VECTOR(0 downto 0);
SIGNAL dpx:STD_LOGIC;
SIGNAL AVG:STD_LOGIC;
begin

-- Differential input

IBUFDS_inst : IBUFDS
generic map (
   DIFF_TERM => FALSE, -- Differential Termination 
   IBUF_LOW_PWR => TRUE, -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
   IOSTANDARD => "DEFAULT")
port map (
   O => D,  -- Buffer output
   I => JB2(0),  -- Diff_p buffer input (connect directly to top-level port)
   IB => JB2(1) -- Diff_n buffer input (connect directly to top-level port)
);

 reset<=btn(0);

 led <= Code(15 downto 8);

-- CLK Divider Nexys 3 Board 100MHz, 10ns
-- EN_1M 100Hz, 10ms; EN_1024 100kHz, 10us
-- EN_256 50kHz,

 CLKDIV0: CLK_DIV port map (
             CLK => CLK1, reset => reset, EN_4 => EN_4, EN_16 => EN_16, 
				 EN_32 => EN_32, EN_64 => EN_64, EN_128 => EN_128, 
				 EN_256 => EN_256, EN_4096 => EN_4096, EN_2048 => EN_2048, 
				 EN_1024 => EN_1024,EN_512 => EN_512, EN_16K => EN_16K, 
				 EN_65K => EN_65K, EN_1M => EN_1M);

-- 7 Segment Display
 dp <= dpx;
 
 SEG0: Segcntrl port map (
             CLK1 => CLK1, Digit0 => Code(3 downto 0), Digit1 => Code(7 downto 4), 
				 Digit2 => Code(11 downto 8), Digit3 => Code(15 downto 12), 
             reset => reset, seg => seg, an => an, dp => dpx);

-- switch (6 downto 5) control number of bits
-- 00  (32)  16
-- 01  (16)  12 
-- 10   (8)  10
-- 11   (2)  8
  JA <= CCode(7 downto 0) when sw(6 downto 5)="00" else
		  CCode(3 downto 0) & "0000" when sw(6 downto 5)="01" else 
		  CCode(1 downto 0) & "000000" when sw(6 downto 5)="01" else 
		  "00000000";
  JC <= CCode(15 downto 8) when sw(6 downto 5)="00" else
		  CCode(11 downto 4) when sw(6 downto 5)="01" else 
		  CCode(9 downto 2) when sw(6 downto 5)="01" else 
		  CCode(7 downto 0);
  Code <= HCode(15 downto 0) when sw(6 downto 5)="00" else
		    HCode(11 downto 0) & "0000" when sw(6 downto 5)="01" else 
		    HCode(9 downto 0) & "000000" when sw(6 downto 5)="01" else 
		    HCode(7 downto 0) & "00000000";

-- switch (4 downto 3) operation
-- 00  SAR operation
-- 01  sawtooth down DAC
-- 10  sawtooth up DAC
-- 11  sine signal DAC
    muxs <= sw(7) & sw(4 downto 3);

    CCode <= SCode(17 downto 2) when muxs = "100" else
	          SCode(16 downto 1) when muxs = "000" else
	          C1Code when muxs = "001" else
	          C1Code when muxs = "101" else
	          not(C1Code) when muxs = "011" else
	          not(C1Code) when muxs = "111" else
	          C1Code;
-- Display
    HCode <= TCode when sw(4 downto 3) = "00" else
	          C1Code when sw(4 downto 3) = "01" else
	          C1Code;

-- Inverted signal for half the cycle
    SigS <= not(sw(7)) & Count8(3);
    C1Code <= BCode when SigS = "00" else
	           BCode when SigS = "10" else
	           not(BCode);
  
-- SAR Code 
   process(CLK1,reset,EN_X, Count8) -- previous EN_T
     begin
       if ( reset = '1') then   
            SCode <="101111111111111111"; -- Start
				MZero <="110111111111111111";
				MOne <= "010000000000000000";
				SSAR <= '1';
       elsif  ((CLK1'event) and CLK1='1') and Count8 = "1010" and EN_X = '1' then 
			 SSAR <= '0';
			 if ( D ='1') then     --
			  	SCode <= (SCode or MOne) and MZero;
          else						 
			  	SCode <= SCode and MZero ;
          end if;
			 if (MZero = "111111111111111110") then
            TCode <= SCode(16 downto 1);
            SCode <="101111111111111111"; -- Start
				MZero <="110111111111111111";
				MOne <= "010000000000000000";
				SSAR <= '1';
			 else 
           MZero <= MZero(0) & MZero (17 downto 1);
           MOne <= MOne(0) & MOne (17 downto 1);
			 end if; 
       end if;
     end process;
	  
	  JB1(4) <= SSAR;
    

-- Counter Code for sawtooth
   process(CLK1,reset,EN_T) -- previous EN_T
     begin
       if ( reset = '1') then   
            BCode<="0000000000000000";
       elsif  ((CLK1'event) and CLK1='1') and EN_T='1' then 
           BCode <= unsigned(BCode) + 1;			   
       end if;
     end process;
    

-- Count8
   process(CLK1,reset,EN_X) -- EN_X
     begin
       if ( reset = '1') then   
            Count8 <= "0000";
       elsif  ((CLK1'event) and CLK1='1') and EN_X='1' then 
           Count8 <= unsigned(Count8) + 1;			   
       end if;
     end process;

-- Setup and hold signal active from 3..6 of 16 cycles
		 SH <= '1' when Count8 = "0010" else 
		       '1' when Count8 = "0011" else-- 16 Times EN_T
		       '1' when Count8 = "0100" else-- 16 Times EN_T
		       '1' when Count8 = "0101" else-- 16 Times EN_T
             '0';
				 
-- Switches control CLK rate of sampling
   -- EN_1M 800us, 16K -> 160us
       Sel <= sw(2 downto 0);
		 EN_X <= EN_65K when Sel="000" else -- 16 Times EN_T
            EN_4096 when Sel="001" else
            EN_1024 when Sel="010" else -- 32 times send
            EN_256 when Sel="011" else -- 8 times
				   -- EN_2048, EN_1024, EN_512
            EN_128 when Sel="100" else -- 4 times
            EN_64 when Sel="101" else -- 2 times
            EN_32 when Sel="110" else -- 1 time
            EN_4096;

		 EN_T <= EN_1M when Sel="000" else  -- 8M x16 needed
            EN_65K when Sel="001" else
            EN_16K when Sel="010" else -- EN_65K x16 4 times send (x32 x8 send)
            EN_4096 when Sel="011" else -- 2 times send
				   -- EN_2048, EN_1024, EN_512
            EN_2048 when Sel="100" else -- 
            EN_1024 when Sel="101" else
            EN_512 when Sel="110" else
            EN_65K;

-- UART implementation
-- Transfer Code
-- Change EN_16K according to sampling rate
-- Counter to send 8 data  x.....<cr><lf>  
   process(CLK1,EN_16K,reset) -- previous EN_16K
     begin
       if ( reset = '1') then   
            ST <="00000";
            STCode <= "0000000000000000";				
       elsif  ((CLK1'event) and CLK1='1') and EN_T='1' then 
           ST<=unsigned(ST) + 1;			   
           if ST = "00001" then
             STCode <= TCode;
			  end if;
       end if;
     end process;
  
  trigger <= ST(0) & EN_16K & "0"; -- tx_full
  wr_uart<= '1' when ( trigger= "110") else
            '0';
				
-- Write x<BCode>crlf
  decode <= ST(3 downto 1);
  i_data <= '0' & STCode(3 downto 0) when decode="101" else
            '0' & STCode(7 downto 4) when decode="100" else
				'0' & STCode(11 downto 8) when decode="011" else
				'0' & STCode(15 downto 12) when decode="010" else
				'0' & "0000" when decode="001" else
			--	"10000" when decode="001" else -- ASCII 0
				"10001" when decode="000" else -- ASCII x
				"10010" when decode="110" else -- ASCII cr
				"10011" when decode="111" else -- ASCII lf
				"10100";

  w_data <= "00110000" when i_data="00000" else   -- HEX 30 ASCII "0"
            "00110001" when i_data="00001" else   -- 1
            "00110010" when i_data="00010" else   -- 2
            "00110011" when i_data="00011" else   -- 3
            "00110100" when i_data="00100" else   -- 4
            "00110101" when i_data="00101" else   -- 5
            "00110110" when i_data="00110" else   -- 6
            "00110111" when i_data="00111" else   -- 7
            "00111000" when i_data="01000" else   -- 8
            "00111001" when i_data="01001" else   -- 9
            "01000001" when i_data="01010" else   -- A
            "01000010" when i_data="01011" else
            "01000011" when i_data="01100" else
            "01000100" when i_data="01101" else
            "01000101" when i_data="01110" else
            "01000110" when i_data="01111" else
            "00110000" when i_data="10000" else  -- 0  
            "01111000" when i_data="10001" else  -- x
				"00001101" when i_data="10010" else  -- cr
				"00001010" when i_data="10011" else  -- lf
				"00100000" when i_data="10100" else  -- space
				"00100000";

  UART0: uart   port map(
      clk=>CLK1, reset=>reset, rd_uart=>'0', wr_uart=>wr_uart, rx=>RXD,
      w_data => w_data, tx_full=> tx_full, rx_empty=> rx_empty,
      r_data => RD_data, tx=> TXI
   );
	
   -- Monitor seriell Interface Connnector JB
	-- Baud Rate 38400 8 Bit N -1  
	-- 16 Bit -> 24 Bit? 1766/s -> 500us (16Bit) -> 30us Maximalfrequenz SAR
	TXD <= TXI;
	JB1(0) <= TXI;
	JB1(1) <= EN_T;
	JB1(2) <= SH;
	JB1(3) <= D;
	
end Behavioral;

